@@ -36,6 +36,7 @@
 #include "../common/sysutils.h" /* (gnupg_fd_t) */
 #include "../common/asshelp.h"  /* (assuan_context_t) */
 #include "../common/i18n.h"
+#include "../common/name-value.h"
 #include "dirmngr-status.h"
 #include "http.h"     /* (parsed_uri_t) */
 
@@ -53,6 +54,7 @@ struct ldap_server_s
   unsigned int starttls:1;       /* Use STARTTLS.  */
   unsigned int ldap_over_tls:1;  /* Use LDAP over an TLS tunnel */
   unsigned int ntds:1;           /* Use Active Directory authentication.  */
+  unsigned int areconly:1;       /* Set LDAP_OPT_AREC_EXCLUSIVE.  */
 };
 typedef struct ldap_server_s *ldap_server_t;
 
@@ -74,6 +76,7 @@ typedef struct fingerprint_list_s *fingerprint_list_t;
 struct fingerprint_list_s
 {
   fingerprint_list_t next;
+  char binlen;  /* If this is not 0 hexfpr actually carries a binary fpr.  */
   char hexfpr[20+20+1];
 };
 
@@ -102,6 +105,7 @@ struct
 
   int force;          /* Force loading outdated CRLs. */
 
+  char *fake_crl;     /* Name of a file with faked CRL entries.  */
 
   unsigned int connect_timeout;       /* Timeout for connect.  */
   unsigned int connect_quick_timeout; /* Shorter timeout for connect.  */
@@ -119,11 +123,23 @@ struct
   int ignore_ocsp_service_url; /* Ignore OCSP service URLs as given in
                                   the certificate.  */
 
+  /* A list of fingerprints of certififcates we should completely
+   * ignore.  These are all stored in binary format.  */
+  fingerprint_list_t ignored_certs;
+
   /* A list of certificate extension OIDs which are ignored so that
      one can claim that a critical extension has been handled.  One
      OID per string.  */
   strlist_t ignored_cert_extensions;
 
+  /* A list of CRL extension OIDs which are ignored so that one can
+   * claim that a critical extension has been handled.  One OID per
+   * string.  */
+  strlist_t ignored_crl_extensions;
+
+  /* Allow expired certificates in the cache.  */
+  int debug_cache_expired_certs;
+
   int allow_ocsp;     /* Allow using OCSP. */
 
   int max_replies;
@@ -143,6 +159,11 @@ struct
                                        current after nextUpdate. */
 
   strlist_t keyserver;              /* List of default keyservers.  */
+
+  const char *user_agent;           /* The HTTP Use-Agent (never NULL).  */
+
+  /* Compatibility flags (COMPAT_FLAG_xxxx).  */
+  unsigned int compat_flags;
 } opt;
 
 
@@ -157,6 +178,7 @@ struct
 #define DBG_NETWORK_VALUE 2048  /* debug network I/O.  */
 #define DBG_LOOKUP_VALUE  8192  /* debug lookup details */
 #define DBG_EXTPROG_VALUE 16384 /* debug external program calls */
+#define DBG_KEEPTMP_VALUE 32768 /* keep some temporary files    */
 
 #define DBG_X509    (opt.debug & DBG_X509_VALUE)
 #define DBG_CRYPTO  (opt.debug & DBG_CRYPTO_VALUE)
@@ -168,6 +190,19 @@ struct
 #define DBG_NETWORK (opt.debug & DBG_NETWORK_VALUE)
 #define DBG_LOOKUP  (opt.debug & DBG_LOOKUP_VALUE)
 #define DBG_EXTPROG (opt.debug & DBG_EXTPROG_VALUE)
+#define DBG_KEEPTMP (opt.debug & DBG_KEEPTMP_VALUE)
+
+/* Compatibility flags */
+
+/* Since version 2.2.12 dirmngr restricted HTTP redirection in an
+ * attempt to mitigate certain CSRF attacks.  It turned out that this
+ * breaks too many WKD deployments and that the attack scenario is not
+ * due to gnupg's redirecting but due to insecure configured systems.
+ * Thus from 2.4.3 on we disable this restriction but allow to use the
+ * old behaviour by using this compatibility flag.  For details see
+ * https://dev.gnupg.org/T6477.  */
+#define COMPAT_RESTRICT_HTTP_REDIR   1
+
 
 /* A simple list of certificate references.  FIXME: Better use
    certlist_t also for references (Store NULL at .cert) */
@@ -178,8 +213,10 @@ struct cert_ref_s
 };
 typedef struct cert_ref_s *cert_ref_t;
 
+/* Forward reference; access only via ks-engine-ldap.c.  */
+struct ks_engine_ldap_local_s;
 
-/* Forward references; access only through server.c.  */
+/* Forward reference; access only through server.c.  */
 struct server_local_s;
 
 #if SIZEOF_UNSIGNED_LONG == 8
@@ -196,6 +233,7 @@ struct server_control_s
   int no_server;      /* We are not running under server control. */
   int status_fd;      /* Only for non-server mode. */
   struct server_local_s *server_local;
+  struct ks_engine_ldap_local_s *ks_get_state;
   int force_crl_refresh; /* Always load a fresh CRL. */
 
   int check_revocations_nest_level; /* Internal to check_revovations.  */
@@ -205,9 +243,12 @@ struct server_control_s
   int audit_events;  /* Send audit events to client.  */
   char *http_proxy;  /* The used http_proxy or NULL.  */
 
+  nvc_t rootdse;     /* Container with the rootDSE properties.  */
+
   unsigned int timeout; /* Timeout for connect calls in ms.  */
 
   unsigned int http_no_crl:1;  /* Do not check CRLs for https.  */
+  unsigned int rootdse_tried:1;/* Already tried to get the rootDSE.  */
 };
 
 
@@ -218,6 +259,7 @@ void dirmngr_deinit_default_ctrl (ctrl_t ctrl);
 void dirmngr_sighup_action (void);
 const char* dirmngr_get_current_socket_name (void);
 int dirmngr_use_tor (void);
+int dirmngr_never_use_tor_p (void);
 
 /*-- Various housekeeping functions.  --*/
 void ks_hkp_housekeeping (time_t curtime);
@@ -225,6 +267,8 @@ void ks_hkp_reload (void);
 void ks_hkp_init (void);
 
 /*-- server.c --*/
+void release_uri_item_list (uri_item_t list);
+
 ldap_server_t get_ldapservers_from_ctrl (ctrl_t ctrl);
 ksba_cert_t get_cert_local (ctrl_t ctrl, const char *issuer);
 ksba_cert_t get_issuing_cert_local (ctrl_t ctrl, const char *issuer);
@@ -251,7 +295,7 @@ gpg_error_t dirmngr_load_swdb (ctrl_t ctrl, int force);
 
 
 /*-- domaininfo.c --*/
-void domaininfo_print_stats (void);
+void domaininfo_print_stats (ctrl_t ctrl);
 int  domaininfo_is_wkd_not_supported (const char *domain);
 void domaininfo_set_no_name (const char *domain);
 void domaininfo_set_wkd_supported (const char *domain);
