@@ -29,12 +29,16 @@ typedef struct {
     size_t maxbuf_size;
 } md_filter_context_t;
 
+typedef struct md_thd_filter_context *md_thd_filter_context_t;
+void md_thd_filter_set_md (md_thd_filter_context_t mfx, gcry_md_hd_t md);
+
 typedef struct {
     int  refcount;          /* Initialized to 1.  */
 
     /* these fields may be initialized */
     int what;		    /* what kind of armor headers to write */
     int only_keyblocks;     /* skip all headers but ".... key block" */
+    int dearmor_mode;       /* dearmor all kind of stuff.  */
     const char *hdrlines;   /* write these headerlines */
 
     /* these fields must be initialized to zero */
@@ -49,6 +53,7 @@ typedef struct {
     int faked;		    /* we are faking a literal data packet */
     int truncated;	    /* number of truncated lines */
     int qp_detected;
+    int dearmor_state;      /* helper for dearmor_mode.  */
     byte eol[3];            /* The end of line characters as a
 			       zero-terminated string.  Defaults
 			       (eol[0]=='\0') to whatever the local
@@ -153,9 +158,9 @@ typedef struct {
 typedef struct {
     char *what;		        /* description */
     u32 last_time;		/* last time reported */
-    unsigned long last;		/* last amount reported */
-    unsigned long offset;	/* current amount */
-    unsigned long total;	/* total amount */
+    uint64_t last;		/* last amount reported */
+    uint64_t offset;	        /* current amount */
+    uint64_t total;	        /* total amount */
     int  refcount;
 } progress_filter_context_t;
 
@@ -163,12 +168,14 @@ typedef struct {
 
 /*-- mdfilter.c --*/
 int md_filter( void *opaque, int control, iobuf_t a, byte *buf, size_t *ret_len);
+int md_thd_filter( void *opaque, int control, iobuf_t a, byte *buf, size_t *ret_len);
 void free_md_filter_context( md_filter_context_t *mfx );
 
 /*-- armor.c --*/
 armor_filter_context_t *new_armor_context (void);
 void release_armor_context (armor_filter_context_t *afx);
 int push_armor_filter (armor_filter_context_t *afx, iobuf_t iobuf);
+int was_armored (armor_filter_context_t *afx);
 int use_armor_filter( iobuf_t a );
 
 /*-- compress.c --*/
